@@ -8,7 +8,7 @@ use Carp qw/croak/;
 use constant PIPE_READ_SIZE => 1024;
 
 use vars qw( $VERSION );
-$VERSION = '1.9';
+$VERSION = '2.000';
 
 BEGIN {
     use Exporter ();
@@ -128,6 +128,14 @@ timeout error may be thrown. Timeout accuracy should be within one second.
 If a timeout does occur, the spawned program will be sent a SIGKILL before
 spawn_safe returns.
 
+=head1 COMPATIBILITY
+
+This module attempts to work on MSWin32 but I've been unable to get it working
+due to strange issues with IO::Select. I haven't been able to track down the
+exact cause, so for now I don't believe this module functions on MSWin32.
+
+Linux and BSD are tested and supported platforms.
+
 =cut
 
 sub spawn_safe {
@@ -218,14 +226,16 @@ sub spawn_safe {
 
     # Don't bother calling time if we're never going to timeout.
     $start_time = defined $timeout ? time() : 1;
-    while ( $sel->count() > 0 ) {
-        foreach my $readme ( $sel->can_read( $timeout ) ) {
+    my $select_time = $timeout;
+    MAIN_WHILE: while ( 1 ) {
+        foreach my $readme ( $sel->can_read( $select_time ) ) {
             # Won't be set if there was a timeout.
             if ( $readme ) {
                 my $read;
                 my $r = sysread( $readme, $read, PIPE_READ_SIZE );
                 if ( ( !defined $r ) || ( $r < 1 ) ) {
                     $sel->remove( $readme );
+                    if ( $sel->count() == 0 ) { last MAIN_WHILE; }
                 } elsif ( $readme == $parent_read_stdout ) {
                     $read_stdout .= $read;
                 } elsif ( $readme == $parent_read_stderr ) {
@@ -249,8 +259,8 @@ sub spawn_safe {
             # timeout as well. Imperfect, but somewhat better than waiting
             # forever. Fortunately this probably won't ever come up.
             my $timenow = time();
-            $timeout -= ( $timenow - $start_time );
-            if ( $timenow < $start_time || $timeout <= 0 ) {
+            $select_time = $timeout - ( $timenow - $start_time );
+            if ( $timenow < $start_time || $select_time <= 0 ) {
                 undef $start_time;
                 last;
             }
@@ -292,6 +302,12 @@ sub spawn_safe {
 
 =head1 CHANGES
 
+=head2 Version 2.000 - 2011-05-12, jeagle
+
+Correct timeout handling. Attempt to correct unit tests for MSWin32, but
+there seems to be an issue with IO::Select preventing it from working
+properly. Update docs for MSWin32.
+
 =head2 Version 1.9 - 2011-05-10, jeagle
 
 Don't use clock_gettime(), use time() and return a timeout if time steps
